@@ -12,6 +12,7 @@
    GNU General Public License for more details.
  */
 
+#include <config.h>
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -27,9 +28,6 @@
 
 #include "parse.h"
 
-/* Default place where the keymaps will be stored */
-#define CFGDIR "/etc/rc_keymaps"
-
 struct input_keymap_entry_v2 {
 #define KEYMAP_BY_INDEX	(1 << 0)
 	u_int8_t  flags;
@@ -141,7 +139,7 @@ static const char args_doc[] =
 
 /* Static vars to store the parameters */
 static char *devclass = "rc0";
-static char *devname = NULL;
+static char *devicename = NULL;
 static int readtable = 0;
 static int clear = 0;
 static int debug = 0;
@@ -196,7 +194,6 @@ static error_t parse_keyfile(char *fname, char **table)
 
 	fin = fopen(fname, "r");
 	if (!fin) {
-		perror("opening keycode file");
 		return errno;
 	}
 
@@ -389,7 +386,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 		period = atoi(arg);
 		break;
 	case 'd':
-		devname = arg;
+		devicename = arg;
 		break;
 	case 's':
 		devclass = arg;
@@ -1386,6 +1383,27 @@ static void show_evdev_attribs(int fd)
 	get_rate(fd, &delay, &period);
 }
 
+static void device_info(int fd, char *prepend)
+{
+	struct input_id id;
+	char buf[32];
+	int rc;
+
+	rc = ioctl(fd, EVIOCGNAME(sizeof(buf)), buf);
+	if (rc >= 0)
+		fprintf(stderr,"%sName: %.*s\n",prepend, rc, buf);
+	else
+		perror ("EVIOCGNAME");
+
+	rc = ioctl(fd, EVIOCGID, &id);
+	if (rc >= 0)
+		fprintf(stderr,
+			"%sbus: %d, vendor/product: %04x:%04x, version: 0x%04x\n",
+			prepend, id.bustype, id.vendor, id.product, id.version);
+	else
+		perror ("EVIOCGID");
+}
+
 static int show_sysfs_attribs(struct rc_device *rc_dev)
 {
 	static struct sysfs_names *names, *cur;
@@ -1410,6 +1428,7 @@ static int show_sysfs_attribs(struct rc_device *rc_dev)
 			display_proto(rc_dev);
 			fd = open(rc_dev->input_name, O_RDONLY);
 			if (fd > 0) {
+				device_info(fd, "\t");
 				show_evdev_attribs(fd);
 				close(fd);
 			} else {
@@ -1431,17 +1450,27 @@ int main(int argc, char *argv[])
 
 	/* Just list all devices */
 	if (!clear && !readtable && !keys.next && !ch_proto && !cfg.next && !test && !delay && !period) {
+		if (devicename) {
+			fd = open(devicename, O_RDONLY);
+			if (fd < 0) {
+				perror("Can't open device");
+				return -1;
+			}
+			device_info(fd, "");
+			close(fd);
+			return 0;
+		}
 		if (show_sysfs_attribs(&rc_dev))
 			return -1;
 
 		return 0;
 	}
 
-	if (cfg.next && (clear || keys.next || ch_proto || devname)) {
+	if (cfg.next && (clear || keys.next || ch_proto || devicename)) {
 		fprintf (stderr, "Auto-mode can be used only with --read, --debug and --sysdev options\n");
 		return -1;
 	}
-	if (!devname) {
+	if (!devicename) {
 		names = find_device(devclass);
 		if (!names)
 			return -1;
@@ -1453,7 +1482,7 @@ int main(int argc, char *argv[])
 		names->name = NULL;
 		free_names(names);
 
-		devname = rc_dev.input_name;
+		devicename = rc_dev.input_name;
 		dev_from_class++;
 	}
 
@@ -1482,29 +1511,45 @@ int main(int argc, char *argv[])
 				cur->fname);
 		if (cur->fname[0] == '/' || ((cur->fname[0] == '.') && strchr(cur->fname, '/'))) {
 			fname = cur->fname;
+			rc = parse_keyfile(fname, &name);
+			if (rc < 0) {
+				fprintf(stderr, "Can't load %s table\n", fname);
+				return -1;
+			}
 		} else {
-			fname = malloc(strlen(cur->fname) + strlen(CFGDIR) + 2);
-			strcpy(fname, CFGDIR);
+			fname = malloc(strlen(cur->fname) + strlen(IR_KEYTABLE_USER_DIR) + 2);
+			strcpy(fname, IR_KEYTABLE_USER_DIR);
 			strcat(fname, "/");
 			strcat(fname, cur->fname);
+			rc = parse_keyfile(fname, &name);
+			if (rc != 0) {
+				fname = malloc(strlen(cur->fname) + strlen(IR_KEYTABLE_SYSTEM_DIR) + 2);
+				strcpy(fname, IR_KEYTABLE_SYSTEM_DIR);
+				strcat(fname, "/");
+				strcat(fname, cur->fname);
+				rc = parse_keyfile(fname, &name);
+			}
+			if (rc != 0) {
+				fprintf(stderr, "Can't load %s table from %s or %s\n", cur->fname, IR_KEYTABLE_USER_DIR, IR_KEYTABLE_SYSTEM_DIR);
+				return -1;
+			}
 		}
-		rc = parse_keyfile(fname, &name);
-		if (rc < 0 || !keys.next) {
-			fprintf(stderr, "Can't load %s table or empty table\n", fname);
+		if (!keys.next) {
+			fprintf(stderr, "Empty table %s\n", fname);
 			return -1;
 		}
 		clear = 1;
 	}
 
 	if (debug)
-		fprintf(stderr, "Opening %s\n", devname);
-	fd = open(devname, O_RDONLY);
+		fprintf(stderr, "Opening %s\n", devicename);
+	fd = open(devicename, O_RDONLY);
 	if (fd < 0) {
-		perror(devname);
+		perror(devicename);
 		return -1;
 	}
 	if (dev_from_class)
-		free(devname);
+		free(devicename);
 	if (get_input_protocol_version(fd))
 		return -1;
 
