@@ -244,6 +244,7 @@ struct parse_key {
 	{"KEY_DISPLAY_OFF", 245},
 	{"KEY_WIMAX", 246},
 	{"KEY_RFKILL", 247},
+	{"KEY_MICMUTE", 248},
 	{"BTN_MISC", 0x100},
 	{"BTN_0", 0x100},
 	{"BTN_1", 0x101},
@@ -303,6 +304,7 @@ struct parse_key {
 	{"BTN_TOOL_FINGER", 0x145},
 	{"BTN_TOOL_MOUSE", 0x146},
 	{"BTN_TOOL_LENS", 0x147},
+	{"BTN_TOOL_QUINTTAP", 0x148},
 	{"BTN_TOUCH", 0x14a},
 	{"BTN_STYLUS", 0x14b},
 	{"BTN_STYLUS2", 0x14c},
@@ -400,6 +402,9 @@ struct parse_key {
 	{"KEY_FRAMEFORWARD", 0x1b5},
 	{"KEY_CONTEXT_MENU", 0x1b6},
 	{"KEY_MEDIA_REPEAT", 0x1b7},
+	{"KEY_10CHANNELSUP", 0x1b8},
+	{"KEY_10CHANNELSDOWN", 0x1b9},
+	{"KEY_IMAGES", 0x1ba},
 	{"KEY_DEL_EOL", 0x1c0},
 	{"KEY_DEL_EOS", 0x1c1},
 	{"KEY_INS_LINE", 0x1c2},
@@ -449,6 +454,15 @@ struct parse_key {
 	{"KEY_NUMERIC_POUND", 0x20b},
 	{"KEY_CAMERA_FOCUS", 0x210},
 	{"KEY_WPS_BUTTON", 0x211},
+	{"KEY_TOUCHPAD_TOGGLE", 0x212},
+	{"KEY_TOUCHPAD_ON", 0x213},
+	{"KEY_TOUCHPAD_OFF", 0x214},
+	{"KEY_CAMERA_ZOOMIN", 0x215},
+	{"KEY_CAMERA_ZOOMOUT", 0x216},
+	{"KEY_CAMERA_UP", 0x217},
+	{"KEY_CAMERA_DOWN", 0x218},
+	{"KEY_CAMERA_LEFT", 0x219},
+	{"KEY_CAMERA_RIGHT", 0x21a},
 	{"BTN_TRIGGER_HAPPY", 0x2c0},
 	{"BTN_TRIGGER_HAPPY1", 0x2c0},
 	{"BTN_TRIGGER_HAPPY2", 0x2c1},
