@@ -686,7 +686,7 @@ int main(int argc, char **argv)
 
 	   VIDIOC_CROPCAP, VIDIOC_G/S_CROP, VIDIOC_G/S_SELECTION
 	   VIDIOC_S_FBUF/OVERLAY
-	   VIDIOC_QBUF/DQBUF/QUERYBUF/PREPARE_BUFS
+	   VIDIOC_QBUF/DQBUF/QUERYBUF/PREPARE_BUFS/EXPBUF
 	   VIDIOC_STREAMON/OFF
 	   */
 
