@@ -13,8 +13,9 @@
 #include <sys/time.h>
 #include <dirent.h>
 #include <math.h>
+#include <config.h>
 
-#ifdef __linux__
+#ifdef HAVE_SYS_KLOG_H
 #include <sys/klog.h>
 #endif
 
@@ -694,7 +695,7 @@ void common_get(int fd)
 
 		if (doioctl(fd, VIDIOC_LOG_STATUS, NULL) == 0) {
 			printf("\nStatus Log:\n\n");
-#ifdef __linux__
+#ifdef HAVE_KLOGCTL
 			len = klogctl(3, buf, sizeof(buf) - 1);
 #endif
 			if (len >= 0) {
