@@ -14,6 +14,7 @@
 #include <sys/mman.h>
 #include <dirent.h>
 #include <math.h>
+#include <config.h>
 
 #include <linux/videodev2.h>
 #include <libv4l2.h>
@@ -24,6 +25,7 @@
 static unsigned stream_count;
 static unsigned stream_skip;
 static unsigned stream_pat;
+static bool stream_loop;
 static unsigned reqbufs_count = 3;
 static char *file;
 
@@ -50,6 +52,8 @@ void streaming_usage(void)
 	       "                     count: the number of buffers to allocate. The default is 3.\n"
 	       "  --stream-from=<file> stream from this file. The default is to generate a pattern.\n"
 	       "                     If <file> is '-', then the data is read from stdin.\n"
+	       "  --stream-loop      loop when the end of the file we are streaming from is reached.\n"
+	       "                     The default is to stop.\n"
 	       "  --stream-pattern=<count>\n"
 	       "                     choose output pattern. The default is 0.\n"
 	       "  --stream-out-mmap=<count>\n"
@@ -148,6 +152,7 @@ static void list_buffers(int fd, unsigned buftype)
 		if (buftype == V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE ||
 		    buftype == V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE) {
 			buf.m.planes = planes;
+			buf.length = VIDEO_MAX_PLANES;
 			memset(planes, 0, sizeof(planes));
 		}
 		if (test_ioctl(fd, VIDIOC_QUERYBUF, &buf))
@@ -167,6 +172,9 @@ void streaming_cmd(int ch, char *optarg)
 	case OptStreamSkip:
 		stream_skip = strtoul(optarg, 0L, 0);
 		break;
+	case OptStreamLoop:
+		stream_loop = true;
+		break;
 	case OptStreamPattern:
 		stream_pat = strtoul(optarg, 0L, 0);
 		stream_pat %= NUM_PATTERNS;
@@ -477,6 +485,29 @@ static void fill_buffer(void *buffer, struct v4l2_pix_format *pix)
 	}
 }
 
+static bool fill_buffer_from_file(void *buffers[], unsigned buffer_lengths[],
+		unsigned buf_index, unsigned num_planes, FILE *fin)
+{
+	for (unsigned j = 0; j < num_planes; j++) {
+		unsigned p = buf_index * num_planes + j;
+		unsigned sz = fread(buffers[p], 1,
+				buffer_lengths[p], fin);
+
+		if (j == 0 && sz == 0 && stream_loop) {
+			fseek(fin, 0, SEEK_SET);
+			sz = fread(buffers[p], 1,
+					buffer_lengths[p], fin);
+		}
+		if (sz == buffer_lengths[p])
+			continue;
+		if (sz)
+			fprintf(stderr, "%u != %u\n", sz, buffer_lengths[p]);
+		// Bail out if we get weird buffer sizes.
+		return false;
+	}
+	return true;
+}
+
 void streaming_set(int fd)
 {
 	if (options[OptStreamMmap] || options[OptStreamUser]) {
@@ -761,6 +792,9 @@ void streaming_set(int fd)
 					}
 				}
 				// TODO fill_buffer_mp(buffers[i], &fmt.fmt.pix_mp);
+				if (fin)
+					fill_buffer_from_file(buffers, buffer_lengths,
+						buf.index, num_planes, fin);
 			} else {
 				buffer_lengths[i] = buf.length;
 				if (is_mmap) {
@@ -775,7 +809,9 @@ void streaming_set(int fd)
 					buffers[i] = calloc(1, buf.length);
 					buf.m.userptr = (unsigned long)buffers[i];
 				}
-				fill_buffer(buffers[i], &fmt.fmt.pix);
+				if (!fin || !fill_buffer_from_file(buffers, buffer_lengths,
+						buf.index, num_planes, fin))
+					fill_buffer(buffers[i], &fmt.fmt.pix);
 			}
 			if (doioctl(fd, VIDIOC_QBUF, &buf))
 				return;
@@ -840,15 +876,9 @@ void streaming_set(int fd)
 				fprintf(stderr, "%s: failed: %s\n", "VIDIOC_DQBUF", strerror(errno));
 				return;
 			}
-			if (fin && !stream_skip) {
-				for (unsigned j = 0; j < num_planes; j++) {
-					unsigned p = buf.index * num_planes + j;
-					unsigned sz = fread(buffers[p], 1,
-							buffer_lengths[p], fin);
-					if (sz != buffer_lengths[p])
-						fprintf(stderr, "%u != %u\n", sz, buffer_lengths[p]);
-				}
-			}
+			if (fin && !fill_buffer_from_file(buffers, buffer_lengths,
+					buf.index, num_planes, fin))
+				break;
 			if (doioctl(fd, VIDIOC_QBUF, &buf))
 				return;
 
@@ -871,10 +901,6 @@ void streaming_set(int fd)
 				}
 			}
 			count++;
-			if (stream_skip) {
-				stream_skip--;
-				continue;
-			}
 			if (stream_count == 0)
 				continue;
 			if (--stream_count == 0)
