@@ -13,6 +13,7 @@
 #include <sys/time.h>
 #include <dirent.h>
 #include <math.h>
+#include <config.h>
 
 #include <linux/videodev2.h>
 #include <libv4l2.h>
@@ -261,6 +262,10 @@ void vidcap_set(int fd)
 								  false);
 				}
 			}
+			/* G_FMT might return a bytesperline value > width,
+			 * reset this to 0 to force the driver to update it
+			 * to the closest value for the new width. */
+			in_vfmt.fmt.pix.bytesperline = 0;
 			if (options[OptSetVideoFormat])
 				ret = doioctl(fd, VIDIOC_S_FMT, &in_vfmt);
 			else
@@ -287,6 +292,11 @@ void vidcap_set(int fd)
 								  true);
 				}
 			}
+			/* G_FMT might return bytesperline values > width,
+			 * reset them to 0 to force the driver to update them
+			 * to the closest value for the new width. */
+			for (unsigned i = 0; i < in_vfmt.fmt.pix_mp.num_planes; i++)
+				in_vfmt.fmt.pix_mp.plane_fmt[i].bytesperline = 0;
 			if (options[OptSetVideoMplaneFormat])
 				ret = doioctl(fd, VIDIOC_S_FMT, &in_vfmt);
 			else
