@@ -30,7 +30,11 @@
 #include <sys/ioctl.h>
 #include <sys/time.h>
 #include <math.h>
+#include <config.h>
+
+#ifdef HAVE_SYS_KLOG_H
 #include <sys/klog.h>
+#endif
 
 #include <linux/videodev2.h>
 #include <media/v4l2-chip-ident.h>
@@ -371,7 +375,7 @@ static int parse_subopt(char **subs, const char * const *subopts, char **value)
 		usage();
 		exit(1);
 	}
-	if (value == NULL) {
+	if (*value == NULL) {
 		fprintf(stderr, "No value given to suboption <%s>\n",
 				subopts[opt]);
 		usage();
@@ -764,11 +768,13 @@ list_done:
 
 	if (options[OptLogStatus]) {
 		static char buf[40960];
-		int len;
+		int len = -1;
 
 		if (doioctl(fd, VIDIOC_LOG_STATUS, NULL, "VIDIOC_LOG_STATUS") == 0) {
 			printf("\nStatus Log:\n\n");
+#ifdef HAVE_KLOGCTL
 			len = klogctl(3, buf, sizeof(buf) - 1);
+#endif
 			if (len >= 0) {
 				char *p = buf;
 				char *q;
